import 'dart:ui';
import 'package:flutter/widgets.dart';
import 'key_book.dart';

export 'key_book.dart';
export 'langs/en_us.dart';
export 'langs/zh_cn.dart';

typedef FLangKeyBookBuilder = FLangKeyBook Function();
typedef FI18nLocaleUpdater = void Function(Locale locale);

/// 翻译字典选项
///
/// [locale] 语言
///
/// [builder] 字典建造器
class FLangKeyBookOption {
  const FLangKeyBookOption(this.locale, this.builder);
  final Locale locale;
  final FLangKeyBookBuilder builder;
}

/// 语言翻译关键字集合
FLangKeyBook get i18nBook => FI18n.ins.currentBook;

/// 国际化
class FI18n {
  FI18n(FLangKeyBook defaultBook, FI18nLocaleUpdater localeUpdater,
      [List<FLangKeyBookOption> otherBookOptions = const []]) {
    _localeUpdater = localeUpdater;
    _buildInstance(defaultBook, otherBookOptions);
  }

  static FI18n? _singleton;

  /// 单例
  static FI18n get ins {
    if (_singleton == null) {
      throw Exception("FI18n尚未创建实例");
    }
    return _singleton!;
  }

  late FLangKeyBook _currentBook;
  late FI18nLocaleUpdater _localeUpdater;
  final Map<Locale, FLangKeyBookBuilder> _bookMap = {};

  /// 当前语言字典
  FLangKeyBook get currentBook => _currentBook;

  /// 当前是否中文
  bool get isCurrentChinese =>
      currentBook.locale.languageCode.toLowerCase() == "zh";

  /// 切换语言
  void switchLanguage([Locale? locale]) {
    if (locale == null) {
      // 顺序找下一个字典
      final keys = _bookMap.keys.toList();
      final curIdx = keys.indexOf(currentBook.locale);
      int nextIdx = curIdx + 1;
      if (nextIdx >= keys.length) nextIdx = 0;
      locale = keys[nextIdx];
    } else {
      if (!_bookMap.keys.contains(locale)) {
        throw Exception("未找到匹配该语言的字典！");
      }
    }
    _localeUpdater.call(locale);
    _currentBook = _bookMap[locale]!.call();
  }

  /// 设置翻译字典(相同语言会覆盖)
  void setKeyBook(Locale locale, FLangKeyBookBuilder bookBuilder) {
    _bookMap[locale] = bookBuilder;
  }

  void _buildInstance(FLangKeyBook defaultBook,
      [List<FLangKeyBookOption> otherBookOptions = const []]) {
    if (_singleton != null) {
      throw Exception("语言实例已存在，请勿重复创建实例！");
    }
    _currentBook = defaultBook;
    _bookMap[defaultBook.locale] = () => defaultBook;
    otherBookOptions.forEach((option) {
      if (_bookMap.containsKey(option.locale)) {
        throw Exception("请检查语言字典集合，不可出现相同的 locale 值！");
      }
      _bookMap[option.locale] = option.builder;
    });
    _singleton = this;
  }

  /// 根据字典树翻译
  ///
  /// [keyTree] 字典树，以.分隔，例：common.loading
  String translate(String keyTree) {
    final tiers = keyTree.split('.');
    dynamic curr = currentBook;
    for (var key in tiers) {
      if (curr is IPropSearchable) {
        curr = curr.getProp(key);
      } else {
        return keyTree;
      }
    }
    return curr;
  }
}
